from card_game_analysis.card import Card
from card_game_analysis.deck import Deck
from card_game_analysis.stack import Stack
from card_game_analysis.player import Player
from card_game_analysis.players import Players



class Player:
    """Player class."""

    def __init__(self):
        self.hand = []

    def add(self, card):
        """Add card to the hand."""
        self.hand.append(card)

    def pull(self, index=0):
        """Pull out a card with a specific index out of the player's hand."""
        card = self.hand[index]
        del self.hand[index]
        return card

    def sort(self):
        """Sort the cards ranging from lowest value to highest value."""
        self.hand.sort(key=lambda card: card.value)

    def show(self, player_name="the player"):
        """Show the cards the player has on their hand in the terminal."""
        print(f"Cards on {player_name}'s hand:\n---------------------------")
        for card in self.hand:
            print(f"Value: {card.value}, Color: {card.color}")

    def size(self):
        """Return amount of cards on the player's hand."""
        return len(self.hand)


if __name__ == "__main__":
    pass



from card_game_analysis.deck import Deck
from card_game_analysis.stack import Stack
from card_game_analysis.players import Players
import itertools

if __name__ == "__main__":
    # Initialize the deck, and shuffle the cards
    deck = Deck()
    deck.shuffle()

    # Initialize the stack
    stack = Stack()

    # Initialize players, and distribute the cards in the deck among them
    number_of_players = 4
    players = Players(number_of_players)
    players.distribute_cards(deck)

    # Sort the cards each player has on their hand
    players.sort()

    players.show()

    # Cycle through the players indefinitely times
    for player_name in itertools.cycle(players.player_names):
        player = players.player_dict[player_name]
        # Add the lowest card in value if stack is empty
        if stack.size() == 0:
            stack.add(player.pull())
            print(player_name)

            # Check if the player has won by getting rid of all their cards
            if player.size() == 0:
                print(f"{player_name} won!")
                break
        else:
            # Find a card that is higher or equal in value to previous card in stack
            for index in range(player.size()):
                if player.hand[index].value >= stack.previous_card().value:
                    stack.add(player.pull(index))
                    print(player_name)
                    break

                # Check if player is not able to play
                if index == player.size() - 1:
                    stack.empty()

            # Check if the player has won by getting rid of all their cards
            if player.size() == 0:
                print(f"{player_name} won!")
                break



from card_game_analysis.deck import Deck
from card_game_analysis.player import Player


class Players:
    """Players class."""

    def __init__(self, number_of_players):
        self.number_of_players = number_of_players
        self.player_names = [
            f"player_{n}" for n in range(1, number_of_players + 1)]
        self.player_dict = {player_name: Player()
                            for player_name in self.player_names}

    def card_split_amount(self):
        """
        Calculate number of cards given to each player
        at the start of the game.
        """
        number_of_players = self.number_of_players
        return int((52 - (52 % number_of_players))/number_of_players)

    def distribute_cards(self, deck):
        """Distribute the cards among the players."""
        player_dict = self.player_dict
        for i in range(self.card_split_amount()):
            for player in player_dict:
                player_dict[player].add(deck.pull())

    def sort(self):
        """
        Sort the cards each player has on their hand
        ranging from lowest value to highest value.
        """
        for player in self.player_dict:
            self.player_dict[player].sort()

    def show(self):
        """Show the cards each player has on their hand in the terminal."""
        for player in self.player_dict:
            self.player_dict[player].show(player)
            print()

    def size(self):
        """Return amount of players."""
        return self.number_of_players


if __name__ == "__main__":
    # Example of use:
    # Initialize the deck, and shuffle the cards
    deck = Deck()
    deck.shuffle()

    # Initialize 5 players, and distribute the cards among the players
    players = Players(5)
    players.distribute_cards(deck)

    # Show remaining cards in the deck
    deck.show()

    # Sort the cards each player has on their hand
    players.sort()

    # Show the cards each player has on their hand
    players.show()



import random
from card_game_analysis.card import Card


class Deck:
    """Deck class."""

    def __init__(self):
        # + ['Jack', 'Queen', 'King', 'Ace']
        values = [n for n in range(2, 15)]
        colors = ['hearts', 'diamonds', 'spades', 'clubs']
        self.cards = [Card(value, color)
                      for value in values for color in colors]

    def shuffle(self):
        """Shuffle the cards in the deck."""
        random.shuffle(self.cards)

    def pull(self):
        """Return a card and remove it from the deck."""
        card = self.cards[0]
        del self.cards[0]
        return card

    def show(self):
        """Show the remaining cards in the deck in the terminal."""
        print("Remaining cards in the deck:\n----------------------------")
        for card in self.cards:
            print(f"Value: {card.value}, Color: {card.color}")

    def size(self):
        """Return amount of remaining cards in the deck."""
        return len(self.cards)


if __name__ == "__main__":
    # Example of use:
    # Initialize the deck, and shuffle the cards
    deck = Deck()
    deck.shuffle()

    # Pull out a card of the deck
    card = deck.pull()
    print(card.value, card.color)

    # Show deck size
    print(deck.size())

    # Show the remaining cards in the deck in the terminal
    deck.show()



class Stack:
    """Stack class."""

    def __init__(self):
        self.cards = []

    def previous_card(self):
        """Return most recently added card."""
        return self.cards[-1]

    def add(self, card):
        """Add a card to the stack."""
        self.cards.append(card)

    def empty(self):
        """Empty the stack."""
        self.cards = []

    def size(self):
        """Return the amount of cards in the stack."""
        return len(self.cards)


if __name__ == "__main__":
    pass
